@@ -1,7 +1,10 @@
 from construct.examples.formats.graphics.emf import emf_file
 from construct.examples.formats.graphics.png import png_file
 from construct.examples.formats.graphics.bmp import bitmap_file
+from construct.examples.formats.graphics.wmf import wmf_file
+from construct.examples.formats.graphics.gif import gif_file
 from construct.examples.formats.filesystem.mbr import mbr_format
 from construct.examples.formats.data.cap import cap_file
 from construct.examples.formats.data.snoop import snoop_file
 from construct.examples.formats.executable.pe32 import pe32_file
+from construct.examples.formats.executable.elf32 import elf32_file
