@@ -1,6 +1,9 @@
-"""
-tcpdump capture file
-"""
+##############################################################
+# WARNING: HEADER IS SKIPPED NOT PARSED, DATETIME CAN BE WRONG
+# 
+# https://wiki.wireshark.org/Development/LibpcapFileFormat
+##############################################################
+
 from construct import *
 import time, datetime
 
@@ -10,7 +13,7 @@ class MicrosecAdapter(Adapter):
         return datetime.datetime.fromtimestamp(obj[0] + obj[1] / 1000000.0)
     def _encode(self, obj, context):
         epoch = datetime.datetime.utcfromtimestamp(0)
-        return [int((obj - epoch).total_seconds()), 0]
+        return [int((obj-epoch).total_seconds()), 0]
 
         # offset = time.mktime(*obj.timetuple())
         # sec = int(offset)
@@ -21,14 +24,7 @@ packet = Struct(
     "time" / MicrosecAdapter(Int32ul >> Int32ul),
     "length" / Int32ul,
     Padding(4),
-    "data" / HexDump(Bytes(this.length)),
+    "data" / Bytes(this.length),
 )
 
-cap_file = Struct(
-    Padding(24),
-    "packets" / GreedyRange(packet),
-)
-
-print("WARNING: header is skipped not parsed, datetime can be wrong")
-print("https://wiki.wireshark.org/Development/LibpcapFileFormat")
-
+cap_file = Padded(24, GreedyRange(packet))
